@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-from SPARQLWrapper import SPARQLWrapper, JSON, XML, N3, RDF
+from SPARQLWrapper import SPARQLWrapper, JSON, XML, N3, RDF, CSV, TSV
 
 sparql = SPARQLWrapper("http://dbpedia.org/sparql")
 sparql.setQuery("""
@@ -35,3 +35,14 @@ sparql.setReturnFormat(RDF)
 results = sparql.query().convert()
 print results.serialize()
 
+# CSV example
+print '\n\n*** CSV Example'
+sparql.setReturnFormat(CSV)
+results = sparql.query().convert()
+print results
+
+# TSV example
+print '\n\n*** TSV Example'
+sparql.setReturnFormat(TSV)
+results = sparql.query().convert()
+print results
